/*
Copyright 2015-2020 Webtrends
SharePoint Social Plugin 
Version: 4.15.0.351
*/

(function () {
    var sp; // Store reference fo webtrends.sp.js file to borrow some fucntions for now

    var spsocial = function (dcs, plugin) {

        // Check if the current page has Social content on it.
        window.WebTrendsSP_intra.prototype.isSocialPage = function () {
            if (document.querySelector("#ms-microfeeddiv")) return true; // Classic Newsfeeds
            else if (document.querySelector("div[class*='commentsWrapper']")) return true; // Modern Newsfeeds
            else if (window.wt_sp_globals.spPageContextInfo.webTemplate === "9") return true; // Classic Blogs
            else if (window.location.href.toLowerCase().indexOf("/hashtagprofile.aspx") >= 0) return true; // My Site Hash Tag Profile
            return false;
        };

        // Skip the rest of the function if social isn't needed, or it isn't social
        if (!sp.cfg.socialMenuClick || sp.cfg.legacySocial || !sp.isSocialPage()) {
            plugin.initialized = false;
            return;
        }

        /*********************************INITIALIZE*********************************/

        // If this isn't initialized because there's no newsfeed on page load, and you move to another page with a newsfeed but using Single Page App behavior
        // feeds on the next page won't be tracked.  Added code to resetFunctionsModernView to keep track of the initialization state and rerun spsocial as needed.
        plugin.initialized = true;

        var impressions = typeof plugin.options.impressions ? plugin.options.impressions : false; // Impressions send one additional hit per visible post
        var trimCharLength = typeof plugin.options.textLength ? plugin.options.textLength : 50; // Trim post text length data
        var replyInProgress = false; // boolean to avoid double hits
        var postInProgress = false; // boolean to avoid double hits
        var feedTitle = ""; // Title of newsfeed filled in later
        var likes = false; // Flag for My Site newsfeed "Likes" tab which doesn't reload posts the same way as others.
        var version = sp.spVersion; // SharePoint version for On Enter logic (Enter vs Shift+Enter)
        var constants = {
            POST: "Post",
            REPLY: "Reply",
            EDIT: "Edit",
            LIKE: "Like",
            UNLIKE: "Unlike",
            EMAIL: "Email a link",
            DELETE: "Delete",
            POST_VIEW: "Post View",
            IMPRESSION: "Impression",
            ARTICLE: "Article",
            SITE_NEWSFEED: "Site Newsfeed",
            MY_SITE_NEWSFEED: "My Site Newsfeed",
            BLOG: "Blog",
            MY_SITE_BLOG: "My Site Blog",
            POST_PAGE: "/post.aspx",
            NEW_POST_PAGE: "/newpost.aspx",
            EDIT_POST_PAGE: "/editpost.aspx",
            EDIT_COMMENT_PAGE: "/editcomment.aspx",
            ALL_COMMENTS_PAGE: "/allcomments.aspx",
            HASH_TAG_PROFILE_PAGE: "/hashtagprofile.aspx",
            DEFAULT_PAGE: "/default.aspx",
            VIEW_HASH_TAG: "View Hashtag",
            FOLLOW_HASH_TAG: "Follow HashTag",
            UNFOLLOW_HASH_TAG: "Unfollow HashTag",
            CLICK_HASH_TAG: "Click HashTag",
            FOLLOW: "Follow",
            STOP_FOLLOWING: "Stop following",
            HASH_FROM_STRING: /#[0-9A-Z]+/ig
        };

        // Determine the site type
        window.WebTrendsSP_intra.prototype.getSocialSiteType = function () {
            var _siteType;
            switch (window.wt_sp_globals.spPageContextInfo.webTemplate) {
                case "1":
                case "64":
                    if ((window.wt_sp_globals.pageDetails && window.wt_sp_globals.pageDetails.PageLayoutType === constants.ARTICLE) ||
                        document.querySelector("aside[data-automation-id='sp-socialbar']")) { 
                        _siteType = constants.ARTICLE;
                    }
                    else {
                        _siteType = constants.SITE_NEWSFEED;
                    }
                    break;
                case "9":
                    var path = window.wt_sp_globals.spPageContextInfo.webServerRelativeUrl.toLowerCase();
                    if (path.indexOf("/my/") >= 0 || path.indexOf("/personal/") >= 0) {
                        _siteType = constants.MY_SITE_BLOG;
                    }
                    else {
                        _siteType = constants.BLOG;
                    }
                    break;
                case "54":
                    _siteType = constants.MY_SITE_NEWSFEED;
                    break;
                default:
                    _siteType = "";
            }
            if (window.wt_sp_globals.socialSiteType !== _siteType) {
                window.wt_sp_globals.socialSiteType = _siteType;
            }
        };

        // Initialize site type
        sp.getSocialSiteType();

        // Get the current visible post count on Page View, for impressions code
        var postCount = 0;
        dcs.addTransform(function (dcsObject, multiTrackObject) {
            try {
                // Update site type on SPA page loads - can't guarantee which addTransform runs first - so run from all
                sp.getSocialSiteType();

                // Check if the page even has a feed
                if (!sp.isSocialPage()) return;

                if (!window.wt_sp_globals.modernView) {
                    // Clasic View
                    if (window.wt_sp_globals.socialSiteType === constants.BLOG || window.wt_sp_globals.socialSiteType === constants.MY_SITE_BLOG) {
                        postCount = document.querySelectorAll("a.ms-accentText").length;
                    }
                    else if (window.wt_sp_globals.socialSiteType === constants.SITE_NEWSFEED || window.wt_sp_globals.socialSiteType === constants.MY_SITE_NEWSFEED) {
                        postCount = document.querySelectorAll("div.ms-microfeed-thread").length;
                    }
                }
                else {
                    // Modern View
                    if (window.wt_sp_globals.socialSiteType === constants.SITE_NEWSFEED || window.wt_sp_globals.socialSiteType === constants.ARTICLE) {
                        postCount = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']").length;
                    }
                }

            }
            catch (ex) { }
        }, 'collect');

        /********************************NEWSFEEDS********************************************/

        // Extract the relevant ID portion of the element id.
        var getIDSubstring = function (el) {
            return el.id.substring(el.id.indexOf("_") + 1, el.id.length);
        };

        // Extract the relevant Post and Reply ID from the element id
        var getPostReplyID = function (id) {
            var result = { success: false };
            if (id) {
                var regex = /\.(\d+\.\d+)\./g;
                var reg = regex.exec(id);
                if (reg && reg[1].indexOf(".") >= 0) {
                    var ids = reg[1].split(".");
                    result.post = ids[0];
                    result.reply = ids[1];
                    result.success = true;
                }
            }
            return result;
        };

        // Get the post container element
        var getPostElement = function (el) {
            var container = null;
            if (!window.wt_sp_globals.modernView) {
                // Classic View
                if (el) {
                    // An existing post, crawl up the DOM to find the parent element.
                    container = sp.getParentByClassNameExact(el, "ms-microfeed-thread") || // Other
                        sp.getParentByClassNameExact(window.wt_sp_globals.tempElement, "ms-microfeed-thread ms-microfeed-greyout"); // Delete
                }
                else {
                    // A brand new post, just grab the first one on the page?
                    // This could be bad if they put two newsfeeds same page?
                    // What if they filter the mysite feed somehow?
                    container = document.querySelector("div.ms-microfeed-thread");
                }
                //	TODO: may need this - extra class for delete : ms-microfeed-thread ms-microfeed-greyout
                //  TODO: may need this - social follow user: "ms-microfeed-confirmationDiv ms-shadow ms-microfeed-rootConfDiv"
            }
            else {
                // Modern View
                if (el) {
                    var id = el.id.substring(el.id.indexOf("reply-") + 6, el.id.length);
                    container = document.querySelector("div[id^='sp-comment-" + id + "']");
                }
                else {
                    // A brand new post, just grab the first one on the page?
                    container = document.querySelector("div[id^='sp-comment-']:not(#sp-comment-input)");
                }
            }
            return container;
        };

        // Get the reply/comment container element
        var getReplyElement = function (el, newReply) {
            var container = null;
            if (el) {
                if (newReply) {
                    // New replies are added to the DOM, find it by checking for the highest list item id.
                    container = getLatestReply(el);
                }
                else {
                    // An existing reply, crawl up the DOM to find the parent element.
                    container = sp.getParentByClassNameExact(el, "ms-microfeed-replyBody") || // Other
                        sp.getParentByClassNameExact(window.wt_sp_globals.tempElement, "ms-microfeed-replyBody"); // Delete
                }
            }
            return container;
        };

        // Get the latest reply for a given post by looking for the highest list item id
        var getLatestReply = function (el) {
            var replies, maxId, replyId, id, ids, itemId;
            if (!window.wt_sp_globals.modernView) {
                // Classic View
                // Collect list of replies for this post based on the post ID
                replies = [].slice.call(document.querySelectorAll("div[threadDivID='" + getIDSubstring(el) + "']"));
                // Reverse the to hopefully place the newest first and avoid extra work
                replies.reverse();
                // Find the reply with the highest list id number
                maxId = 0;
                replyId = "";
                for (var i = 0; i < replies.length; i++) {
                    // Get the list item id from the id string
                    id = getIDSubstring(replies[i]);
                    ids = getPostReplyID(id);
                    if (ids.success) {
                        itemId = parseInt(ids.reply) || 0;
                        // Keep track of the highest item id
                        if (itemId > maxId) {
                            replyId = id;
                            maxId = itemId;
                        }
                    }
                }
                return document.querySelector("[id='ms-post_" + replyId + "']");
            }
            else {
                // Modern View
                // Collect list of replies and keep the highest number
                var par = sp.getParentByClassNameContains(el, "spComment_");
                if (par) replies = [].slice.call(par.querySelectorAll("div[data-automation-id='sp-comment-block-reply']"));
                else replies = [].slice.call(document.querySelectorAll("div[data-automation-id='sp-comment-block-reply']"));
                // Find the reply with the highest list id number
                maxId = 0;
                replyId = "";
                for (var j = 0; j < replies.length; j++) {
                    // Get the list item id from the id string
                    id = parseInt(replies[j].id.replace("sp-comment-", ""));
                    // Keep track of the highest item id
                    if (id > maxId) {
                        replyId = id;
                        maxId = id;
                    }
                }
                return document.querySelector("div[id='sp-comment-" + replyId + "']");
            }
        };

        // Initialize and get the post id, text, author, image, and create the proper URL
        var getPostData = function (el) {
            var res = {};
            res.id = "Post ID Not Found";
            res.body = "Post Text Not Found";
            res.author = "Post Author Not Found";
            res.image = false;
            res.hash = "";
            var id, url;
            if (el) {
                if (!window.wt_sp_globals.modernView) {
                    // Classic View
                    // TODO: Need alternative method for id for My Site Blog Posts
                    id = el.hasAttribute("attachid") ? el.getAttribute("attachid") : getIDSubstring(el);
                    if (id) {
                        // Collect ID of post
                        res.id = id;
                        // Collect data from post element
                        res = getData(id, res);
                        // Append URL
                        url = getURL(el, id);
                        res.dcssip = url.dcssip;
                        res.dcsuri = url.dcsuri;
                        res.dcsqry = url.dcsqry;
                        res.dcsref = url.dcsref;
                    }
                }
                else {
                    // Modern View
                    id = el.hasAttribute("id") ? el.getAttribute("id") : "";
                    if (id) {
                        // Collect ID of post
                        res.id = id;
                        // Collect data from post element
                        res = getData(id, res);
                        // Append URL
                        url = getURL(el, id);
                        res.dcssip = url.dcssip;
                        res.dcsuri = url.dcsuri;
                        res.dcsqry = url.dcsqry;
                        res.dcsref = url.dcsref;
                    }
                }
            }
            return res;
        };

        // Initialize and get the reply/comment id, text, author, and image
        var getReplyData = function (el) {
            var res = {};
            res.id = "";
            res.body = "";
            res.author = "";
            res.login = "";
            res.image = false;
            res.hash = "";
            if (el) {
                if (!window.wt_sp_globals.modernView)
                    res = getData(getIDSubstring(el), res);
                else 
                    res = getData(el.id, res);    
            }
            return res;
        };

        // Get the information from the post or reply container
        var getData = function (id, res) {
            var textEl, authorEl, loginEl, account, login, imgEl, text, nodes;
            res.id = id;
            res.hash = "";
            if (!window.wt_sp_globals.modernView) {
                // Classic View
                // Collect post/comment text
                // TODO: Check up on this - If someone is @ in the text, it could be a problem for privacy options.
                // Possible fix is to Check for a.ms-subtleLink#ms-actorElement within ms-postbody, 
                // And the then use.replace on each name found within?
                textEl = document.querySelector("span[id^='ms-postBody_" + id + "']");
                if (textEl) {
                    text = "";
                    // Encrypt @user names if desired
                    if (sp.cfg.obfuscate && textEl.querySelector("a.ms-subtleLink[href*='accountname=']")) {
                        nodes = textEl.childNodes;
                        for (var i = 0; i < nodes.length; i++) {
                            if (!nodes[i].tagName) {
                                // Regular text
                                text += " " + sp.getElementText(nodes[i]);
                            }
                            else {
                                // @User Name
                                text += sp.dcsEncryption(sp.getElementText(nodes[i])).toString();
                            }
                        }
                        res.body = text.trim();
                    }
                    else res.body = sp.getElementText(textEl);

                    // Extract Hash data from text
                    res.hash = getHash(res.body);
                    res.hashId = getHashIdAll(textEl);

                    // Trim text body after collection of other data
                    res.body = trimText(res.body);
                }

                // Collect post/comment author name
                authorEl = document.querySelector("[id^='ms-authorlabel_" + id + "']");
                if (authorEl) {
                    res.author = sp.getElementText(authorEl);
                }
                // Collect post/comment author login
                loginEl = document.querySelector("[id^='ms-authorlabel_" + id + "']");
                // SP2016 and SPO, the <a> is on a child element, not the ms-authorlabel_
                if (loginEl && loginEl.tagName !== "A") loginEl = loginEl.querySelector("a");
                if (loginEl) {
                    account = sp.getQueryParamValue("accountname", loginEl.getAttribute("href"));
                    if (account) {
                        login = decodeURIComponent(account);
                        // Online login users pipe delimited string
                        if (login.indexOf("|") >= 0) login = login.split("|").pop();
                        res.login = login;
                    }
                }
                // Determine if post/comment has image
                imgEl = document.querySelector("img[id^='ms-microfeed-attachmentImage_" + id + "']");
                if (imgEl) {
                    res.image = true;
                    if (!textEl) res.body = "(Image Only)";
                }
            }
            else {
                // Modern View
                //if (sp.spVersion === "2019") { // TODO: Uncomment after merging 433 which updates spVersion to show 2019 as well
                if (sp.spVersion !== "SPO") {
                    // Collect post/comment text
                    textEl = document.querySelector("div#" + id + " div[class^='spText_'] span span");
                    if (textEl) {
                        // Can't @ users in 2019, just get the text
                        res.body = trimText(sp.getElementText(textEl));
                    }
                    // Collect post/comment author name
                    authorEl = document.querySelector("div#" + id + " span[class^='spAuthor_']");
                    if (authorEl) {
                        res.author = sp.getElementText(authorEl);
                    }
                // Can't retrieve login from Modern Posts
                // Can't post images in Modern Posts
                }
                else if (sp.spVersion === "SPO") {
                    // Collect post/comment text
                    textEl = document.querySelector("div#" + id + " div[class^='spText_'] span[class^='commentRichText_']");
                    if (textEl) {
                        text = "";
                        // Encrypt @user names if desired
                        if (sp.cfg.obfuscate && textEl.querySelector("a[data-sp-mention-user-id]")) {
                            nodes = textEl.childNodes;
                            for (var j = 0; j < nodes.length; j++) {
                                if (!nodes[j].tagName) {
                                    // Regular text
                                    text += " " + sp.getElementText(nodes[j]);
                                }
                                else {
                                    // @User Name
                                    text += " @" + sp.dcsEncryption(sp.getElementText(nodes[j]).replace(/^@/, "")).toString();
                                }
                            }
                            res.body = trimText(text.trim());
                        }
                        else res.body = trimText(sp.getElementText(textEl));
                    }
                    // Collect post/comment author name
                    authorEl = document.querySelector("div#" + id + " span[class^='spAuthor_']");
                    if (authorEl) {
                        res.author = sp.getElementText(authorEl);
                    }
                // Can't retrieve login from Modern Posts
                // Can't post images in Modern Posts
                }
            }            
            return res;
        };

         // Create the proper URL for a direct link to the post
        var getURL = function (elem, id) {
            var res = {};
            res.dcssip = window.location.hostname;
            res.dcsuri = (typeof asyncDeltaManager === "undefined") ? window.location.pathname : window.location.pathname + window.location.hash;
            res.dcsqry = window.location.search;
            res.dcsref = window.location.href;
            if (elem) {
                if (!window.wt_sp_globals.modernView) {
                    // Classic View
                    var uri = window.wt_sp_globals.spPageContextInfo.webServerRelativeUrl;
                    if (uri.lastIndexOf("/") === uri.length - 1) uri = uri.substring(0, uri.length - 1);
                    if (window.wt_sp_globals.socialSiteType === constants.MY_SITE_NEWSFEED) {
                        res.dcssip = window.location.hostname;
                        res.dcsuri = uri + "/ThreadView.aspx";
                        res.dcsqry = "?ThreadID=" + id;
                    }
                    else if (window.wt_sp_globals.socialSiteType === constants.SITE_NEWSFEED) {
                        res.dcssip = window.location.hostname;
                        res.dcsuri = uri + "/NewsFeed.aspx";
                        res.dcsqry = "?ThreadID=" + id;
                    }
                }
                else {
                    // Modern View
                    if (window.wt_sp_globals.socialSiteType === constants.SITE_NEWSFEED || window.wt_sp_globals.socialSiteType === constants.ARTICLE) {
                        res.dcsuri += "#" + id;
                        res.dcsqry = "";
                    }
                }
                if (window.location.port) res.dcssip += ":" + window.location.port;
            }
            return res;
        };

        // Check if post or reply has an image
        var postHasImage = function (el) {
            if (!el) return false;
            var newPost = sp.getParentByIDContains(el, "ms-microblogdiv");
            if (newPost && newPost.querySelector("img.ms-microfeed-imageAttachmentPreview")) return true;
            else return false;
        };


        // Check if post or reply has an image
        var replyHasImage = function (el) {
            if (!el) return false;
            var reply = sp.getParentByIDContains(el, "ms-replyInputFocus");
            if (reply && reply.querySelector("img.ms-microfeed-imageAttachmentPreview")) return true;
            else return false;
        };

        // Check if the post text is empty, or too long, which will throw an error in SP
        var isValid = function (el, type) {
            if (!el) return false;
            var text;
            if (!window.wt_sp_globals.modernView) {
                // Classic View
                text = el.value ? el.value : "";
                // SP2013 limited to 512
                if (version === "2013") {
                    // Text not empty , less than 513, and not just whitespace
                    if (text.length > 0 && text.length < 513 && text.trim() !== "") return true;
                    // No text, but an image
                    else if (type === constants.POST && postHasImage(el) && text.length < 513) return true;
                    // No text, but an image
                    else if (type === constants.REPLY && replyHasImage(el) && text.length < 513) return true;
                }
                // SP2016 and SPO can do seemingly unlimited length
                else {
                    // Text not empty , and not just whitespace
                    if (text.length > 0 && text.trim() !== "") return true;
                    // No text, but an image
                    else if (type === constants.POST && postHasImage(el)) return true;
                    // No text, but an image
                    else if (type === constants.REPLY && replyHasImage(el)) return true;
                }
            }
            else {
                // Modern View
                text = sp.getElementText(el);
                // Text not empty, not too long, and not just whitespace
                if (text.length > 0 && text.length < 2001 && text.trim() !== "") return true;
                // Can't attach images in Modern View
            }
            return false;
        };

        // Trim post text length to a specified amount for reporting
        var trimText = function (text) {
            // Trim post title length
            if (text && trimCharLength && text.length > trimCharLength) { // Only happens if the string is longer than the limit we defined
                text = text.substring(0, trimCharLength);  // Trims up to a certain number of characters
                text = text.substring(0, text.lastIndexOf(" ")) + "..."; // Trims on the last index of a space, then adds "..."
            }
            return text;
        };

        // Send the data to Webtrends
        var sendData = function (post, reply, action, actionTarget, webpart, navigation) {
            if (!post || !action) return;
            if (likes && post.getAttribute("displayInMyLikesView") === "false") return;
            if (!webpart) webpart = "";
            if (!navigation) navigation = "";
            var dl = (action === constants.IMPRESSION) ? "100" : "SHP";
            var title = (action === constants.IMPRESSION) ? (constants.POST + " " + constants.IMPRESSION) : ("Link: " + action);
            var postData = getPostData(post);
            var replyData = getReplyData(reply);
            var followAuthor = "";
            var followAuthorLogin = "";
            var a = action.toLowerCase();
            if (a.indexOf("follow") >= 0 && a.indexOf("follow up") === -1) {
                followAuthor = replyData.author ? replyData.author : postData.author;
                followAuthorLogin = replyData.login ? replyData.login : postData.login;
            }
            var art, art_url, art_author, art_action = "";
            if (window.wt_sp_globals.socialSiteType === constants.ARTICLE) {
                art = document.title;
                art_url = window.location.protocol + "//" + window.location.hostname + window.location.pathname;
                art_author = sp.getElementText(document.querySelector("div[class^='personaDetails_'] div[class^='primaryText_']"));
                art_action = actionTarget + " " + action;
                if (action === constants.POST) {
                    art_action = constants.POST;
                }
            }
            window.Webtrends.multiTrack({
                argsa: [
                    "DCS.dcssip", postData.dcssip,
                    "DCS.dcsuri", postData.dcsuri,
                    "DCS.dcsqry", postData.dcsqry,
                    "DCS.dcsref", postData.dcsref,
                    "WT.ti", title,
                    "WT.dl", dl,
                    "WT.soc_content", feedTitle,
                    "WT.soc_feed_type", window.wt_sp_globals.socialSiteType,
                    "WT.soc_url", window.location.protocol + "//" + postData.dcssip + postData.dcsuri + postData.dcsqry,
                    "WT.soc_text", postData.body,
                    "WT.soc_author", postData.author,
                    "WT.soc_login", postData.login,
                    "WT.soc_action", action,
                    "WT.soc_image", postData.image,
                    "WT.soc_type", actionTarget,
                    "WT.soc_reply_text", replyData.body,
                    "WT.soc_reply_image", replyData.image,
                    "WT.soc_reply_author", replyData.author,
                    "WT.soc_reply_login", replyData.login,
                    "WT.soc_follow_author", followAuthor,
                    "WT.soc_follow_login", followAuthorLogin,
                    "WT.soc_hash", (!reply ? postData.hash : replyData.hash),
                    "WT.soc_hash_id", (!reply ? postData.hashId : replyData.hashId),
                    "WT.article", art,
                    "WT.article_url", art_url,
                    "WT.article_author", art_author,
                    "WT.article_action", art_action,
                    "WT.shp_wpv", webpart,
                    "WT.nv", navigation
                ],
                finish: function (dcsObject, multiTrackObject) {
                    sp.finishCleanup(dcsObject, multiTrackObject);
                }
            });
        };

        // Impression functions

        // Triggers the sending of Posts for impressions after links that change entire list
        // My Site Newsfeed links - Following, Everyone, Mentions
        // Does not run on "SHOW MORE POSTS"
        var triggerImpressions = function (dcsObject, options) {
            var event = options.event;
            var iterations = 0;
            var postsViewChange = setInterval(function (dcsObject) {
                if (iterations > 20) clearInterval(postsViewChange); // Quit trying after 10 seconds
                iterations++;
                // Wait until page is finished processing
                if (document.querySelector("div.ms-microfeed-processingDiv.ms-visibilityHidden")) {
                    clearInterval(postsViewChange);
                    setTimeout(function () {
                        var posts = document.querySelectorAll("div.ms-microfeed-thread");
                        for (var i = 0; i < posts.length; i++) {
                            var post = posts[i];
                            var reply = null;
                            if (!post.querySelector("div.ms-microfeed-oneLiner,div.ms-microfeed-activityText")) { // Ignore "notice" posts
                                var navigation = dcsObject ? dcsObject.dcsNavigation(event, dcsObject.navigationtag) : null;
                                sendData(post, reply, constants.IMPRESSION, constants.POST, sp.getWebPartTitleOnClick(post), navigation);
                            }
                        }
                        postCount = posts.length;
                        likes = false;
                    }, 1500);
                }
            }, 500);
        };

        // Post and Reply functions 
        // Trigger a Post, Reply, or Delete event on "Enter" key
        var triggerOnEnter = function (e) {
            try {
                if (!e) e = window.event;
                if (e.keyCode === 13) {
                    var par;
                    var element = (e.target || e.srcElement);
                    var id = element.id;
                    if (!window.wt_sp_globals.modernView) {
                        // Classic View
                        // Delete just needs an Enter event
                        if (id.indexOf("ms-confConfirmButton") === 0) {
                            var ids = getPostReplyID(id);
                            if (ids.success && ids.post === ids.reply) triggerDelete(e, window.wt_sp_globals.tempElement, constants.POST);
                            else triggerDelete(e, window.wt_sp_globals.tempElement, constants.REPLY);
                        }
                        // SP2013 can post/reply on enter.  Others needs Shift + Enter
                        else if ((version === "2013" && !e.shiftKey) || (version !== "2013" && e.shiftKey)) {
                            if (id.indexOf("ms-microbloginputbox") === 0) {
                                // Avoid double posts
                                if (postInProgress) return;
                                else if (isValid(element, constants.POST)) triggerNewPost(e, element, null);
                            }
                            else if (id.indexOf("ms-replyinput") === 0) {
                                // Avoid double posts
                                if (replyInProgress) return false;
                                else {
                                    par = sp.getParentByClassNameContains(element, "ms-microfeed-replyArea");
                                    if (par && isValid(par.querySelector("textarea"), constants.REPLY)) triggerNewReply(e, element, null);
                                }
                            }
                        }
                    }
                    else {
                        // Modern View
                        // Delete just needs and enter event
                        if (element.hasAttribute("data-automation-id") && element.getAttribute("data-automation-id") === "yesButton") {
                            // This is too slow, and the elements are removed from the DOM before a hit fires.
                            if (window.wt_sp_globals.tempElement2) {
                                var post = window.wt_sp_globals.tempElement2;
                                var reply;
                                if (post.getAttribute("data-automation-id") === "sp-comment-block-reply") {
                                    reply = post;
                                    post = sp.getParentByClassNameContains(post.parentNode, "spComment_");
                                }
                                sendData(post, reply, constants.DELETE, reply ? constants.REPLY : constants.POST, sp.getWebPartTitleOnClick(element), null);
                            }
                        }
                        // Post/Reply needs Ctrl + Enter
                        if (e.ctrlKey) {
                            if (id === "sp-comment-input") {
                                // Avoid double posts
                                if (postInProgress) return;
                                else if (isValid(element, constants.POST)) triggerNewPost(e, element, null);
                            }
                            else if (id.indexOf("sp-comment-input-reply-") === 0) {
                                // Avoid double posts
                                if (replyInProgress) return false;
                                else {
                                    par = sp.getParentByClassNameContains(element, "spComment_");
                                    if (par && isValid(element, constants.REPLY)) triggerNewReply(e, element, null);
                                }
                            }
                        }

                    }
                }
            }
            catch (ex) {
                postInProgress = false;
            }
        };

        // Wait for a pending Post to finish, then send the data of the topmost post, and update the current post count
        var triggerNewPost = function (e, el, dcsObject) {
            try {
                if (!e || !el) return;
                postInProgress = true;
                var navigation = dcsObject ? dcsObject.dcsNavigation(e, dcsObject.navigationtag) : null;
                // classic = ms-microfeed-thread, modern = div[data-automation-id='sp-comment-block-root']
                var previousCount = document.querySelectorAll("div.ms-microfeed-thread,div[data-automation-id='sp-comment-block-root']").length;
                var count = 0;
                var waitForPost = setInterval((function () {
                    try {
                        // If there's a SharePoint error pop up, or it's gone over 60 seconds trying to post, just quit
                        if (document.querySelector("#ms-errorDialogDiv") || count > 240) {
                            clearInterval(waitForPost);
                            postInProgress = false;
                        }
                        else {
                            var elems, post, reply;
                            if (!window.wt_sp_globals.modernView) {
                                // Classic View
                                // Send a hit if "post pending" is gone, and the post count has gone up.
                                elems = document.querySelectorAll("div.ms-microfeed-thread");
                                if ((elems.length > previousCount && !document.querySelector("#ms-post_pending"))) {
                                    clearInterval(waitForPost);
                                    post = getPostElement(null);
                                    reply = null;
                                    sendData(post, reply, constants.POST, constants.POST, sp.getWebPartTitleOnClick(el), navigation);
                                    postCount = document.querySelectorAll("div.ms-microfeed-thread").length;
                                    postInProgress = false;
                                }
                            }
                            else {
                                // Modern View
                                // Send a hit if "post pending" is gone, and the post count has gone up.
                                elems = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']");
                                if ((elems.length > previousCount && !document.querySelector("div[id*='sp-comment-optimistic']"))) {
                                    clearInterval(waitForPost);
                                    post = getPostElement(null);
                                    reply = null;
                                    sendData(post, reply, constants.POST, constants.POST, sp.getWebPartTitleOnClick(el), navigation);
                                    postCount = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']").length;
                                    postInProgress = false;
                                }
                            }

                        }
                        count++;
                    }
                    catch (ex) {
                        clearInterval(waitForPost);
                        postInProgress = false;
                    }
                }), 250);
            }
            catch (ex) {
                postInProgress = false;
            }
        };

        // Reply event handler
        var triggerNewReply = function (e, el, dcsObject) {
            try {
                replyInProgress = true;
                var count = 0;
                var post, reply;
                var waitForReply = setInterval(function () {
                    if (!window.wt_sp_globals.modernView) {
                        // If there's a SharePoint error pop up, or it's gone over 60 seconds trying to post, just quit
                        if (document.querySelector("#ms-errorDialogDiv") || count > 240) {
                            clearInterval(waitForReply);
                            replyInProgress = false;
                        }
                        // Check for the error dialogue or still pending reply in SPO - check on SP2016 for this?
                        else if (!document.querySelector("#ms-content_pending")) {
                            clearInterval(waitForReply);
                            post = getPostElement(el);
                            reply = getReplyElement(el, true);
                            sendData(post, reply, constants.REPLY, constants.POST, sp.getWebPartTitleOnClick(el), (dcsObject ? dcsObject.dcsNavigation(e, dcsObject.navigationtag) : null));
                            setTimeout(function () {
                                replyInProgress = false;
                            }, 500);
                        }
                    }
                    else {
                        // Modern View
                        // Check for pending reply
                        if (!document.querySelector("div[id*='sp-comment-optimistic']")) {
                            clearInterval(waitForReply);
                            post = getPostElement(el);
                            reply = getReplyElement(el, true);
                            sendData(post, reply, constants.REPLY, constants.POST, sp.getWebPartTitleOnClick(el), (dcsObject ? dcsObject.dcsNavigation(e, dcsObject.navigationtag) : null));
                            setTimeout(function () {
                                replyInProgress = false;
                            }, 500);
                        }
                    }
                    count++;
                }, 250);
            }
            catch (ex) {
                replyInProgress = false;
            }
        };

        // Delete event handler
        var triggerDelete = function (e, el, type) {
            var post = getPostElement(el);
            var reply = getReplyElement(el, false);
            sendData(post, reply, constants.DELETE, type, sp.getWebPartTitleOnClick(el), null);
        };

        // Newsfeed Post Impressions
        if (impressions) {
            // Initial Post Impressions on Page View - Triggered by Collect event, which should allow this to work on MDS pages as well.
            dcs.addTransform(function (dcsObject, multiTrackObject) {
                try {
                    // Update site type on SPA page loads - can't guarantee which addTransform runs first - so run from all
                    sp.getSocialSiteType();

                    // Check if the page even has a feed
                    if (!sp.isSocialPage()) return;

                    if (!window.wt_sp_globals.modernView) {
                        // Classic VIew
                        // Check if this is a thread view and abort, since the Page View will send it as a "Post View"?
                        if (!sp.getQueryParamValue("ThreadID", window.location.search)) {
                            setTimeout(function () {
                                var posts = document.querySelectorAll("div.ms-microfeed-thread");
                                for (var i = 0; i < posts.length; i++) {
                                    var post = posts[i];
                                    var reply = null;
                                    if (!post.querySelector("div.ms-microfeed-oneLiner,div.ms-microfeed-activityText")) { // Ignore "notice" posts
                                        sendData(post, reply, constants.IMPRESSION, constants.POST, sp.getWebPartTitleOnClick(post), null);
                                    }
                                }
                            }, 250);
                        }
                    }
                    else {
                        // Modern View
                        setTimeout(function () {
                            var posts = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']");
                            for (var i = 0; i < posts.length; i++) {
                                var post = posts[i];
                                var reply = null;
                                sendData(post, reply, constants.IMPRESSION, constants.POST, sp.getWebPartTitleOnClick(post), null);
                            }
                        }, 250);
                    }
                }
                catch (ex) { }
            }, 'collect');

            // Classic View: SHOW MORE POSTS link - triggers for loop to send the additional impressions excluding those already sent.
            dcs.addSelector("button#ms-MoreThreadsButtonLabel", {
                actionElems: { 'BUTTON': 1 },
                filter: function (dcsObject, options) {
                    var e = options.event;
                    if (dcsObject._isRightClick(e)) return true;
                    postCount = document.querySelectorAll("div.ms-microfeed-thread").length;
                    var iterations = 0;
                    var showMorePosts = setInterval(function () {
                        var posts = document.querySelectorAll("div.ms-microfeed-thread");
                        if (iterations > 20 || (posts.length > postCount && document.querySelector("div.ms-microfeed-processingDiv.ms-visibilityHidden"))) {
                            clearInterval(showMorePosts);
                            for (var i = postCount; i < posts.length; i++) { // Start loop at previous postCount to avoid sending same post twice
                                var post = posts[i];
                                var reply = null;
                                if (!post.querySelector("div.ms-microfeed-oneLiner,div.ms-microfeed-activityText")) { // Ignore "notice" posts
                                    // Check if viewing a likes page to avoid too many impressions from hidden data in the HTML
                                    if (sp.translateLanguage(sp.getElementText(document.querySelector("a.ms-pivotControl-surfacedOpt-selected"))) === "Likes") likes = true;
                                    sendData(post, reply, constants.IMPRESSION, constants.POST, sp.getWebPartTitleOnClick(post), null);
                                }
                            }
                            postCount = posts.length;
                            likes = false;
                        }
                        iterations++;
                    }, 500);
                    return true;
                }
            });

            // Modern View: Show more comments - triggers for loop to send the additional impressions excluding those already sent.
            dcs.addSelector("button.ms-Link[class*='spMore_']", {
                actionElems: { 'BUTTON': 1 },
                filter: function (dcsObject, options) {
                    var e = options.event;
                    if (dcsObject._isRightClick(e)) return true;
                    postCount = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']").length;
                    var iterations = 0;
                    var showMorePosts = setInterval(function () {
                        var posts = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']");
                        if (iterations > 20 || posts.length > postCount) {
                            clearInterval(showMorePosts);
                            setTimeout(function () {
                                for (var i = postCount; i < posts.length; i++) { // Start loop at previous postCount to avoid sending same post twice
                                    var post = posts[i];
                                    var reply = null;
                                    sendData(post, reply, constants.IMPRESSION, constants.POST, sp.getWebPartTitleOnClick(post), null);
                                }
                                postCount = posts.length;
                            }, 1000);
                        }
                        iterations++;

                    }, 500);
                    return true;
                }
            });

            // Following/Everyone/Mentions links on My Site Newsfeed which shows alternative posts
            dcs.addSelector("a.ms-pivotControl-surfacedOpt", {
                actionElems: { 'A': 1, 'DIV': 1 },
                filter: function (dcsObject, options) {
                    var element = options.element;
                    var event = options.event;
                    if (dcsObject._isRightClick(event)) return true;
                    if (sp.translateLanguage(sp.getElementText(element)) === "Likes") likes = true;
                    else likes = false;
                    triggerImpressions(dcsObject, options);
                    return true;
                }
            });

            // My Site Newsfeed ECB Menu options, overflow menu of links above (Following,Everyone,Mentions,Likes,Activities)
            dcs.addSelector("div#ms-microfeeddiv a.ms-pivotControl-overflowDot", {
                filter: function (dcsObject, options) {
                    if (dcsObject._isRightClick(options.e)) return true;
                    (setTimeout(function () {
                        var obj = dcsObject;
                        var elems = document.querySelectorAll("div#ms-microfeeddiv a.ms-core-menu-link");
                        for (var i = 0; i < elems.length; i++) {
                            var elem = elems[i];
                            if (!elem.wtBound) {
                                // What about IE8?
                                elem.addEventListener(sp.setBindingOption(), function (e) {
                                    var element = e.target || e.srcElement;
                                    if (sp.translateLanguage(sp.getElementText(element)) === "Likes") likes = true;
                                    else likes = false;
                                    triggerImpressions(obj, options);
                                }, false);
                                elem.wtBound = true;
                            }
                        }
                    }, 1000));
                    return true;
                }
            });
        }

        // Newsfeed Post Interactions

        // New Post (Click)
        dcs.addSelector("button#checkPost,button#ms-postbutton,button.ms-Button.ms-Button--primary", {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                var element = options.element;
                var event = options.event;
                // Avoid right clicks, disabled links, or double clicks
                if (dcsObject._isRightClick(event) || element.disabled || postInProgress) return true;
                var par, textarea;
                if (!window.wt_sp_globals.modernView) {
                    // Classic
                    par = sp.getParentByIDContains(element, "ms-microblogdiv");
                    if (par) {
                        textarea = par.querySelector("textarea");
                        if (isValid(textarea, constants.POST)) {
                            triggerNewPost(event, textarea, dcsObject);
                        }
                    }
                }
                else {
                    // Modern
                    // Abort for non social buttons
                    if (!sp.getParentByClassNameContains(element, "spComments_")) return true;
                    par = sp.getParentByClassNameContains(element, "spReply_");
                    if (par) {
                        textarea = par.querySelector("div#sp-comment-input");
                        if (isValid(textarea, constants.POST)) {
                            triggerNewPost(event, textarea, dcsObject);
                        }
                    }
                }
                return true;
            }
        });

        // New Post/Reply/Delete (On Enter)
        if (window.addEventListener) {
            document.addEventListener("keydown", triggerOnEnter, true);
        }
        else {
            // TODO: Not working in MSIE8 ?
            document.attachEvent("onkeydown", triggerOnEnter);
        }

        // Classic View: Like/Unlike a Post (Click)
        dcs.addSelector('a[id^="ms-likebutton_"]', {
            filter: function (dcsObject, options) {
                var el = options.element;
                var e = options.event;
                if (dcsObject._isRightClick(event)) return true;
                var action = !el.getAttribute("liked") ? constants.LIKE : constants.UNLIKE;
                var post = getPostElement(el);
                var reply = getReplyElement(el, false);
                if (reply) {
                    // Reply Like/Unlike
                    sendData(post, reply, action, constants.REPLY, sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                else {
                    // Post Like/Unlike
                    sendData(post, reply, action, constants.POST, sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                return true;
            }
        });

        // Modern View: Like/Unlike a Post (Click)
        dcs.addSelector('button.ms-Button.ms-Button--action', {
            actionElems: {'BUTTON': 1},
            filter: function (dcsObject, options) {
                var el = options.element;
                var e = options.event;
                if (dcsObject._isRightClick(event)) return true;
                if (!el.querySelector("i[data-icon-name='Like']") && !el.querySelector("i[data-icon-name='LikeSolid']")) return true;
                var action = el.querySelector("i[data-icon-name='Like']") ? constants.LIKE : constants.UNLIKE;
                var post = sp.getParentByClassNameContains(el, "spComment_");
                var reply;
                if (post) {
                    // If the parent container is a sp-comment-block-reply, go up one more container for the parent post
                    if (post.getAttribute("data-automation-id") === "sp-comment-block-reply") {
                        reply = post;
                        post = sp.getParentByClassNameContains(post.parentNode, "spComment_");
                    }
                    if (reply) {
                        // Reply Like/Unlike
                        sendData(post, reply, action, constants.REPLY, sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                    }
                    else {
                        // Post Like/Unlike
                        sendData(post, reply, action, constants.POST, sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                    }
                }
                return true;
            }
        });

        // Social Follow User
        dcs.addSelector('a[id*="followlinkfori"],a[id*="FollowLinkFor"]', {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                var el = options.element;
                var e = options.event;
                var action;
                var text = sp.translateLanguage(sp.getElementText(el)).toLowerCase(); 
                if (text.indexOf("stop") === 0) action = "Stop Following User";
                else action = "Follow User";
                var post = getPostElement(el);
                var reply = getReplyElement(el, false);
                if (reply) {
                    sendData(post, reply, action, constants.REPLY, sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                else {
                    sendData(post, reply, action, constants.POST, sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                return true;
            }
        });

        // Reply to a Post (Click)
        dcs.addSelector('button[id^="checkReply"],button[id^="ms-postreplybutton_"],button.ms-Button.ms-Button--primary', {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                try {
                    var e = options.event;
                    var el = options.element;
                    // Avoid right clicks, disabled links, or double clicks
                    if (dcsObject._isRightClick(e) || el.disabled || replyInProgress) return true;
                    var par;
                    if (!window.wt_sp_globals.modernView) {
                        // Classic View
                        par = sp.getParentByClassNameContains(el, "ms-microfeed-replyArea");
                        if (par && isValid(par.querySelector("textarea"), constants.REPLY)) triggerNewReply(e, el, dcsObject);
                    }
                    else {
                        // Modern
                        // Abort for non social buttons
                        if (!sp.getParentByClassNameContains(el, "spComments_")) return true;
                        par = sp.getParentByClassNameContains(el, "spReply_");
                        if (par) {
                            var textarea = par.querySelector("div[id^='sp-comment-input-reply-']");
                            if (isValid(textarea, constants.REPLY)) {
                                triggerNewReply(e, textarea, dcsObject);
                            }
                        }
                    }
                }
                catch (err) {
                    replyInProgress = false;
                }
                return true;
            }
        });

        // Classic View: Delete button - Stores a reference to the delete button for triggerOnEnter to find the post/reply
        dcs.addSelector("button.ms-microfeed-deleteButton", {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                window.wt_sp_globals.tempElement = options.element;
                return true;
            }
        });

        // Classic View: Delete a reply or comment (Click)
        dcs.addSelector('button.ms-microfeed-confirmationDivButton[id^="ms-confConfirmButton_"]', {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                var el = options.element;
                var e = options.event;
                if (dcsObject._isRightClick(e)) return true;
                // Get data for deleted post/reply
                var post = getPostElement(el);
                var reply = getReplyElement(el, false);
                if (reply) {
                    sendData(post, reply, constants.DELETE, constants.REPLY, sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                else {
                    sendData(post, reply, constants.DELETE, constants.POST, sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                // Update post count for impressions
                var postCountAfterDeletion = setInterval(function () {
                    if (document.querySelector(".ms-microfeed-processingDiv.ms-visibilityHidden")) {
                        clearInterval(postCountAfterDeletion);
                        postCount = document.querySelectorAll("div.ms-microfeed-thread").length;
                    }
                }, 500);
                // Exit
                return true;
            }
        });

        // Modern View: Delete button - Stores a reference to the delete button for deletions to track the proper link
        dcs.addSelector("button.ms-Button.ms-Button--action.ms-Button--command[class*='deleteAction']", {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                // Store the reply or post temporalily
                // Use .tempElement2 because .tempElement is used by webtrends.sp.js to store the subsequent ECB link and overwrites this.
                window.wt_sp_globals.tempElement2 = sp.getParentByClassNameContains(options.element, "spComment_");
                return true;
            }
        });

        // Modern View: Delete a reply or comment (Click)
        // dcs.addSelector('button.ms-ContextualMenu-link[data-automation-id="comment-delete-button"]', {  
        // This works on the ecb Delete link, but the confirm is preferable - confirm button may not have access to reply or comment if it's deleted too fast
        dcs.addSelector('button.ms-Button.ms-Button--primary[data-automation-id="yesButton"]', {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                var e = options.event;
                if (dcsObject._isRightClick(e)) return true;
                if (!window.wt_sp_globals.tempElement2) return true;
                var post = window.wt_sp_globals.tempElement2;
                var reply;
                if (post) {
                    // If the parent container is a sp-comment-block-reply, go up one more container for the parent post
                    if (post.getAttribute("data-automation-id") === "sp-comment-block-reply") {
                        reply = post;
                        post = sp.getParentByClassNameContains(post.parentNode, "spComment_");
                    }
                    if (reply) {
                        // Reply Like/Unlike
                        sendData(post, reply, constants.DELETE, constants.REPLY, sp.getWebPartTitleOnClick(post), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                    }
                    else {
                        // Post Like/Unlike
                        sendData(post, reply, constants.DELETE, constants.POST, sp.getWebPartTitleOnClick(post), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                    }
                }
                // Update post count for impressions
                setTimeout(function () {
                    postCount = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']").length;
                }, 500);
                delete window.wt_sp_globals.tempElement2;
                return true;
            }
        });

        // Site Newsfeed ECB Menu options
        dcs.addSelector("a.ms-core-menu-root[id*=microfeed-moreDropDownMenu]", {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.e)) return true;
                var waitForDropDown = (setInterval(function () {
                    var elems = document.querySelectorAll("a.ms-core-menu-link");
                    if (elems.length > 0) {
                        clearInterval(waitForDropDown);
                        for (var i = 0; i < elems.length; i++) {
                            var elem = elems[i];
                            if (!elem.wtBound) {
                                // What about IE8?
                                elem.addEventListener(sp.setBindingOption().click, function (e) {
                                    var obj = dcsObject;
                                    var el = e.target || e.srcElement;
                                    var text = sp.translateLanguage(sp.getElementText(el));
                                    // Delete is handled with confirmation pop up exit
                                    if (text === constants.DELETE) {
                                        return false;
                                    }
                                    // Continue - get text boxes
                                    var post = getPostElement(el);
                                    var reply = getReplyElement(el, false);
                                    var regex = new RegExp(constants.FOLLOW + "|" + constants.STOP_FOLLOWING, "i");
                                    if (text.match(regex)) { 
                                        // Social Follow # links
                                        var getLinkEl = function (el, postBox) {
                                            var element = el;
                                            var hashLinks = postBox.querySelectorAll("a.ms-subtleLink#ms-actorElement[href*='HashTagProfile']");
                                            for (var i = 0; i < hashLinks.length; i++) {
                                                var innerText = sp.getElementText(hashLinks[i]);
                                                if (hash === innerText) {
                                                    element = hashLinks[i];
                                                    break;
                                                }
                                            }
                                            return element;
                                        };
                                        // Get hash info
                                        var a = sp.getParentByTagName(el, "A");
                                        regex = new RegExp("^" + constants.FOLLOW, "i");
                                        var follow = text.match(regex) ? true : false;
                                        var hash = text.match(constants.HASH_FROM_STRING)[0];
                                        var link = getLinkEl(a, reply || post);
                                        var res = sp.getURIArrFromHREF(link.getAttribute("href"));
                                        Webtrends.multiTrack({
                                            element: a,
                                            argsa: [
                                                "DCS.dcssip", res.dcssip,
                                                "DCS.dcsuri", res.dcsuri,
                                                "DCS.dcsqry", res.dcsqry,
                                                "DCS.dcsref", res.dcsref,
                                                "WT.ti", "Link: " + text,
                                                "WT.dl", "SHP",
                                                "WT.soc_action", follow ? constants.FOLLOW_HASH_TAG : constants.UNFOLLOW_HASH_TAG,
                                                "WT.soc_hash", hash,
                                                "WT.soc_hash_id", getHashId(link)
                                            ],
                                            finish: function (dcsObject, multiTrackObject) {
                                                sp.finishCleanup(dcsObject, multiTrackObject);
                                            }
                                        });
                                    }
                                    else {
                                        // All other ECB options
                                        if (reply) {
                                            sendData(post, reply, text, constants.REPLY, sp.getWebPartTitleOnClick(el), obj.dcsNavigation(e, obj.navigationtag));
                                        }
                                        else {
                                            sendData(post, reply, text, constants.POST, sp.getWebPartTitleOnClick(el), obj.dcsNavigation(e, obj.navigationtag));
                                        }
                                    }

                                }, false);
                                elem.wtBound = true;
                            }
                        }
                    }
                }, 250));
                return true;
            }
        });

        // My Site Newsfeed "MENTIONS" tab "Show conversation" link
        dcs.addSelector("a.ms-microfeed-activityPostLink", {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.e)) return true;
                var id = getIDSubstring(options.element);
                var waitForConversation = (setInterval(function () {
                    var el = document.querySelector("[id*='" + id + "']");
                    if (el) {
                        clearInterval(waitForConversation);
                        var par = sp.getParentByClassNameExact(el, "ms-microfeed-thread");
                        var post = getPostElement(par);
                        sendData(post, null, constants.POST_VIEW, constants.POST, sp.getWebPartTitleOnClick(par), null); // dcsNavigation fails on this event
                    }
                }, 250));
                return true;
            }
        });

        /*********************************NEWSFEED HASHTAGS*********************************/

        // Check if this is a hashtag profile page
        var isHashTagProfilePage = function () {
            return (window.location.href.toLowerCase().indexOf(constants.HASH_TAG_PROFILE_PAGE) >= 0);
        };

        // Get the hash text from the header element of /HashTagProfile.aspx pages
        var getHashFromHeader = function () {
            var result = "";
            var pageTitle = document.querySelector("#pageTitle");
            if (pageTitle) {
                var text = sp.getElementText(pageTitle);
                if (text.match(constants.HASH_FROM_STRING)) {
                    result = text.match(constants.HASH_FROM_STRING)[0];
                }
            }
            return result;
        };

        // Get all the social hash texts from a post or reply
        var getHash = function (text) {
            var result = "";
            var hashMatch = text.match(constants.HASH_FROM_STRING);
            if (hashMatch) {
                result = hashMatch.join(";");
            }
            return result;
        };

        // Get the social hash ID from the links TermID parameter
        var getHashId = function (a) {
            if (!a || !a.hasAttribute("href")) return "";
            return sp.getQueryParamValue("TermID", a.getAttribute("href"));
        };

        // Get all the social hash IDs from a post or reply's text
        var getHashIdAll = function (textEl) {
            var result = "";
            var anchorLinks = textEl.querySelectorAll("a.ms-subtleLink#ms-actorElement");
            var hashIds = [];
            for (var i = 0; i < anchorLinks.length; i++) {
                hashIds.push(getHashId(anchorLinks[i]));
            }
            result = hashIds.join(";");
            return result;
        };

        // Get the hash ID from the element "name" attribute
        var getHashIdFromName = function (textEl) {
            var result = "";
            var name = textEl.getAttribute("name");
            if (name) {
                var hashId = name.split(".")[3];
                if (hashId) {
                    hashId = Array.from(hashId);
                    hashId.splice(8, 0, "-");
                    hashId.splice(13, 0, "-");
                    hashId.splice(18, 0, "-");
                    hashId.splice(23, 0, "-");
                    hashId = hashId.join("");
                    result = hashId;
                }
            }
            return result;
        };

        // Add soc_hash parameters to the page view of a HasthTagProfile page
        dcs.addTransform(function (dcsObject, multiTrackObject) {
            try {
                if (isHashTagProfilePage()) {
                    multiTrackObject.argsa.push(
                        "WT.soc_action", constants.VIEW_HASH_TAG,
                        "WT.soc_hash", getHashFromHeader(),
                        "WT.soc_hash_id", sp.getQueryParamValue("TermID", window.location.href)
                    );
                }
            }
            catch (ex) { }
        }, "collect");

        // Clear out stuck soc_hash parameters on clicks from a HashTagProfile page
        dcs.addTransform(function (dcsObject, multiTrackObject) {
            try {
                if (isHashTagProfilePage()) {
                    multiTrackObject.argsa.push(
                        "WT.soc_action", "",
                        "WT.soc_hash", "",
                        "WT.soc_hash_id", ""
                    );
                }
            }
            catch (ex) { }
        }, "multitrack");

        // Social "follow this #tag" on a HashTagProfile page
        dcs.addSelector('a[id*="HashTagProfile_FollowTagLinkOption"],a#HashTagProfile_UnfollowTagLinkOption', {
            filter: function (dcsObject, options) {
                // Abort right clicks
                if (dcsObject._isRightClick(options.event)) return true;

                // Abort on other pages
                if (!isHashTagProfilePage()) return true;

                // Continue
                return false;
            },
            transform: function (dcsObject, multiTrackObject) {
                var el = multiTrackObject.element;

                // Get action
                var action = constants.FOLLOW_HASH_TAG;
                if (el.getAttribute("isfollowed") === "True") {
                    action = constants.UNFOLLOW_HASH_TAG;
                }

                // Get the URL of the hash
                var res = sp.getURIArrFromHREF(window.location.href);

                multiTrackObject.argsa.push(
                    "DCS.dcssip", res.dcssip,
                    "DCS.dcsuri", res.dcsuri,
                    "DCS.dcsqry", res.dcsqry,
                    "DCS.dcsref", res.dcsref,
                    "DCS.dcsref", window.location.href,
                    "WT.ti", "Link: " + sp.getElementText(el),
                    "WT.dl", "SHP",
                    "WT.soc_action", action,
                    "WT.soc_hash", getHashFromHeader(),
                    "WT.soc_hash_id", sp.getQueryParamValue("TermID", window.location.href),
                    "WT.nv", dcsObject.dcsNavigation(multiTrackObject.event, dcsObject.navigationtag)
                );
            },
            finish: function (dcsObject, multiTrackObject) {
                sp.finishCleanup(dcsObject, multiTrackObject);
            }
        });

        // Social "Follow #Hash" below a post or reply (ECB tracked in ExecuteOnClick wrapper below)
        dcs.addSelector('a[id^="ms-followbutton_"][name^="FollowButtonFor"]', {
            filter: function (dcsObject, options) {
                // Abort right clicks
                if (dcsObject._isRightClick(options.event)) return true;

                // Abort clicks to similar elements
                if (sp.getElementText(options.element).indexOf("#") === -1) return true;

                // Continue
                return false;
            },
            transform: function (dcsObject, multiTrackObject) {
                var el = multiTrackObject.element;

                // Get link text
                var title = sp.getElementText(el);

                // Get action taken on the link
                var action;
                var actionText = sp.translateLanguage(title).toLowerCase();
                if (actionText.indexOf("stop") === 0) action = constants.UNFOLLOW_HASH_TAG;
                else action = constants.FOLLOW_HASH_TAG;

                // Get hash info
                var hashId = getHashIdFromName(el);

                // Get the URL of the hash if possible
                var res = sp.getURIArrFromHREF(window.location.href);
                var par = sp.getParentByClassNameHas(el, "ms-microfeed-rootBody,ms-microfeed-replyMessage");
                if (par) {
                    var body = par.querySelector(".ms-microfeed-replyBody,.ms-microfeed-postBody");
                    if (body) {
                        // Find matching <a> link from post text - works in SP2013. SP2016, SP2019, and SPO don't seem to have a URL in the element
                        var hashes = body.querySelectorAll("a#ms-actorElement.ms-subtleLink");
                        for (var i = 0; i < hashes.length; i++) {
                            var href = hashes[i].getAttribute("href");
                            if (sp.getQueryParamValue("TermID", href) === hashId) {
                                res = sp.getURIArrFromHREF(href.replace(":80/", "/"));
                                break;
                            }
                        }
                    }
                }

                multiTrackObject.argsa.push(
                    "DCS.dcssip", res.dcssip,
                    "DCS.dcsuri", res.dcsuri,
                    "DCS.dcsqry", res.dcsqry,
                    "DCS.dcsref", res.dcsref,
                    "DCS.dcsref", window.location.href,
                    "WT.ti", title,
                    "WT.dl", "SHP",
                    "WT.soc_action", action,
                    "WT.soc_hash", getHash(title),
                    "WT.soc_hash_id", hashId,
                    "WT.nv", dcsObject.dcsNavigation(multiTrackObject.event, dcsObject.navigationtag)
                );
            },
            finish: function (dcsObject, multiTrackObject) {
                sp.finishCleanup(dcsObject, multiTrackObject);
            }
        });

        // Social "#Hash" click in a post or reply
        dcs.addSelector('a#ms-actorElement.ms-subtleLink', {
            filter: function (dcsObject, options) {
                // Abort right clicks
                if (dcsObject._isRightClick(options.event)) return true;
                // Abort clicks to similar elements
                var el = options.element;
                if (el.hasAttribute("href") && el.getAttribute("href").indexOf("HashTagProfile.aspx") === -1) return true;
                return false;
            },
            transform: function (dcsObject, multiTrackObject) {
                var el = multiTrackObject.element;
                var res = sp.getURIArrFromHREF(el.getAttribute("href"));
                var title = sp.getElementText(el);
                multiTrackObject.argsa.push(
                    "DCS.dcssip", res.dcssip,
                    "DCS.dcsuri", res.dcsuri,
                    "DCS.dcsqry", res.dcsqry,
                    "DCS.dcsref", res.dcsref,
                    "WT.ti", title,
                    "WT.dl", "SHP",
                    "WT.soc_action", constants.CLICK_HASH_TAG,
                    "WT.soc_hash", getHash(title),
                    "WT.soc_hash_id", getHashId(el),
                    "WT.nv", dcsObject.dcsNavigation(multiTrackObject.event, dcsObject.navigationtag)
                );
            },
            finish: function (dcsObject, multiTrackObject) {
                sp.finishCleanup(dcsObject, multiTrackObject);
            }
        });

        // Store reference to the ... menu for ECB menu clicks to track Follow #Hash - processed with anonymous ExecuteOnClick wrapper below
        dcs.addSelector('a[id*="ms-mysitemenu-control_Link_microfeed-moreDropDownMenu_"].ms-core-menu-root', {
            filter: function (dcsObject, options) {
                (setTimeout(function () {
                    window.wt_sp_globals.tempElement = options.element;
                }, 500));
                return true;
            }
        });

        /*********************************BLOGS***************************************/

        // Collect data for Blog posts
        var getPostDataBlog = function (el) {
            var post = {};
            post.title = "Post Title Not Found";
            post.body = "Post Text Not Found";
            post.author = "Post Author Not Found";
            post.category = "Post Category Not Found";

            // Get data from post on a feed or post page
            var getDataFromExistingPost = function (el, original) {
                var result = original;

                // Get the post title
                var getTitle = function (el) {
                    var title = "";
                    var elem = el.querySelector("h1 a,h2 a");
                    if (elem) title = sp.getElementText(elem);
                    return title;
                };

                // Get the post body text
                var getTextBody = function (el) {
                    var body = "";
                    var elem = el.querySelector("div[class='ms-blog-postBody']");
                    if (elem) body = trimText(sp.getElementText(elem));
                    return body;
                };

                // Get the post author
                var getAuthor = function (el) {
                    var author = "";
                    var elem = el.parentNode.querySelector("a.ms-subtleLink");
                    if (elem) author = sp.getElementText(elem);
                    return author;
                };

                // Get the post author login
                var getAuthorLogin = function (el) {
                    var login = "";
                    // TODO: Is this possible?
                    return login;
                };

                // Get the post categories
                var getCategories = function (el) {
                    var categories = [];
                    var elems = el.querySelectorAll("#blgcat");
                    for (var i = 0; i < elems.length; i++) {
                        categories.push(sp.getElementText(elems[i]));
                    }
                    return categories.join(";");
                };

                // Get the URL of the post
                var getURL = function (el) {
                    var href;
                    var a = el.querySelector("h1 a,h2 a");
                    if (a) href = a.getAttribute("href");
                    var url = sp.getURIArrFromHREF(href);
                    return url;
                };

                // Collect post data
                result.title = getTitle(el);
                result.body = getTextBody(el);
                result.author = getAuthor(el);
                result.login = getAuthorLogin();
                result.category = getCategories(el);
                var url = getURL(el);
                result.dcssip = url.dcssip;
                result.dcsuri = url.dcsuri;
                result.dcsqry = url.dcsqry;
                result.dcsref = url.dcsref;
                return result;
            };

            // Get data from the NewPost.aspx form
            var getDataFromNewPost = function (original) {
                var result = original;
                // Collect post data
                result = getBlogFormData();
                // Append URL
                var path = window.location.pathname.toLowerCase().replace(constants.NEW_POST_PAGE, constants.POST_PAGE);
                // Can't get the ID value of the post in SP2013 until after page refresh
                // Page doesn't redirect to new post, so there's no reliable way to get the ID without doing API calls
                var url = sp.getURIArrFromHREF(window.location.hostname + path);
                result.dcssip = url.dcssip;
                result.dcsuri = url.dcsuri;
                result.dcsqry = url.dcsqry;
                result.dcsref = url.dcsref;
                return result;
            };

            // Get data from the EditPost.aspx form
            var getDataFromEditPost = function (original) {
                var result = original;
                // Collect post data
                result = getBlogFormData();
                // Append URL
                var dom = window.location.hostname;
                var uri = window.wt_sp_globals.spPageContextInfo.serverRequestPath.toLowerCase().replace(constants.EDIT_POST_PAGE, constants.POST_PAGE);
                var qry = "?ID=" + sp.getQueryParamValue("ID", window.location.search);
                var url = sp.getURIArrFromHREF(dom + uri + qry);
                result.dcssip = url.dcssip;
                result.dcsuri = url.dcsuri;
                result.dcsqry = url.dcsqry;
                result.dcsref = url.dcsref;
                return result;
            };

            // Get Data from the EditComment.aspx form
            var getDataFromEditComment = function (original) {
                var result = original;
                if (window.wt_sp_globals.blogPostData) {
                    // Collect Post info from API call - can't get from Edit Comments page otherwise
                    result = window.wt_sp_globals.blogPostData;
                }
                // Append URL
                var url = sp.getURIArrFromHREF(result.url || decodeURIComponent(sp.getQueryParamValue("Source", window.location.search)));
                result.dcssip = url.dcssip;
                result.dcsuri = url.dcsuri;
                result.dcsqry = url.dcsqry;
                result.dcsref = url.dcsref;
                return result;
            };

            // Collect data
            if (el) {
                // Existing post on feed or post page
                post = getDataFromExistingPost(el, post);
            }
            else if (window.location.href.toLowerCase().indexOf(constants.NEW_POST_PAGE) >= 0) {
                // New post
                post = getDataFromNewPost(post);
            }
            else if (window.location.href.toLowerCase().indexOf(constants.EDIT_POST_PAGE) >= 0) {
                // Edit post
                post = getDataFromEditPost(post);
            }
            else if (window.location.href.toLowerCase().indexOf(constants.EDIT_COMMENT_PAGE) >= 0) {
                // Edit comment
                post = getDataFromEditComment(post);
            }
            return post;
        };

        // Collect data for blog comments
        var getReplyDataBlog = function (el) {
            var reply = {};
            reply.id = "";
            reply.body = "";
            reply.author = "";
            reply.login = "";

            // Collect reply text
            var getReplyText = function (el) {
                var text = "";
                var elem = el.querySelector("textarea");
                if (elem) text = trimText(elem.value || sp.getElementText(elem));
                return text;
            };

            // Collect reply author
            var getReplyAuthor = function () {
                var author = "";
                if (window.wt_sp_user && window.wt_sp_user["WT.shp_uname"]) {
                    author = window.wt_sp_user["WT.shp_uname"];
                }
                return author;
            };

            // Collect reply login
            var getReplyLogin = function () {
                var login = "";
                if (window.wt_sp_user && window.wt_sp_user["WT.shp_login"]) {
                    login = window.wt_sp_user["WT.shp_login"];
                }
                return login;
            };

            // Collect data
            if (el) {
                // New reply
                reply.body = getReplyText(el);
                reply.author = getReplyAuthor();
                reply.login = getReplyLogin();
            }
            else if (window.location.href.toLowerCase().indexOf(constants.EDIT_COMMENT_PAGE) >= 0) {
                // Editing a reply
                reply = getBlogFormData();
                reply.author = getReplyAuthor();
                reply.login = getReplyLogin();
            }
            return reply;
        };

        // Get the data from the EditForm pages for posts and comments
        var getBlogFormData = function () {
            var form = {};
            form.title = "";
            form.body = "";
            form.author = "";
            form.category = "";

            // Clean up HTML and whitespace
            var cleanText = function (text) {
                return text.replace(/(<([^>]+)>)/ig, "").trim();
            };

            // Get the post/comment title
            var getTitle = function () {
                var title = "";
                var elem = document.querySelector("input[id^='Title_']");
                if (elem) {
                    // Strip HTML and whitespace from value attribute
                    title = cleanText(elem.value);
                }
                return title;
            };

            // Get the post/comment body
            var getTextBody = function () {
                var body = "";
                // Edit/New Post/Comment body, New Comment body
                var elem = document.querySelector("div[id^='Body_'][role='textbox'],textarea[id^='Body_']");
                if (elem) {
                    if (elem.value) {
                        // Strip HTML and whitespace from value attribute
                        body = cleanText(elem.value);
                    }
                    else {
                        // Get the inner text
                        body = sp.getElementText(elem);
                    }
                }
                return trimText(body);
            };

            // Get the post/comment author
            var getAuthor = function () {
                var author = "";
                // "Created by" Author link
                var elem = document.querySelector("a.ms-peopleux-userdisplink.ms-subtleLink");
                if (elem) {
                    // Grab the subteLink inner text
                    author = sp.getElementText(elem);
                }
                else if (window.location.href.toLowerCase().indexOf(constants.NEW_POST_PAGE) >= 0) {
                    // New post, try to use current user from User Profile Service API
                    if (window.window.wt_sp_user && window.window.wt_sp_user["WT.shp_uname"]) {
                        author = window.window.wt_sp_user["WT.shp_uname"];
                    }
                }
                return author;
            };

            // Get the post/comment author login
            var getAuthorLogin = function () {
                var login = "";
                if (window.location.href.toLowerCase().indexOf(constants.NEW_POST_PAGE) >= 0) {
                    if (window.wt_sp_user && window.wt_sp_user["WT.shp_login"]) {
                        login = window.wt_sp_user["WT.shp_login"];
                    }
                }
                return login;
            };

            // Get the post/comment category
            var getCategories = function () {
                var categories = [];
                // Categories table
                var elems = document.querySelectorAll("table[id^='PostCategory_'] select[id$='_SelectResult'] option");
                for (var i = 0; i < elems.length; i++) {
                    // Append inner text of each selectedcategory element
                    categories.push(sp.getElementText(elems[i]));
                }
                return categories.join(";");
            };

            // Collect form data
            form.title = getTitle();
            form.body = getTextBody();
            form.author = getAuthor();
            form.login = getAuthorLogin();
            form.category = getCategories();
            return form;
        };

        // Get appropriate LI container for posts
        var getBlogPostContainer = function (el) {
            var post = sp.getParentByTagName(el, "LI");
            if (post && !post.querySelector(".ms-blog-postBody")) {
                // link may be in it's own sublist, need the parent list
                post = sp.getParentByTagName(post.parentNode, "LI");
            }
            return post;
        };

        // Send the data to Webtrends for Blog entries
        var sendDataBlog = function (post, reply, action, actionTarget, webpart, navigation) {
            if ((!post && (action !== constants.EDIT && action !== constants.POST && action !== constants.DELETE)) || !action) return;
            if (!webpart) webpart = "";
            if (!navigation) navigation = "";
            var dl = (action === constants.IMPRESSION) ? "100" : "SHP";
            var title = (action === constants.IMPRESSION) ? (constants.POST + " " + constants.IMPRESSION) : ("Link: " + action);
            var postData = getPostDataBlog(post);
            var replyData = getReplyDataBlog(reply);
            window.Webtrends.multiTrack({
                argsa: [
                    "DCS.dcssip", postData.dcssip,
                    "DCS.dcsuri", postData.dcsuri,
                    "DCS.dcsqry", postData.dcsqry,
                    "DCS.dcsref", postData.dcsref,
                    "WT.ti", title,
                    "WT.dl", dl,
                    "WT.soc_content", feedTitle,
                    "WT.soc_feed_type", window.wt_sp_globals.socialSiteType,
                    "WT.soc_url", window.location.protocol + "//" + postData.dcssip + postData.dcsuri + postData.dcsqry,
                    "WT.soc_title", postData.title,
                    "WT.soc_text", postData.body,
                    "WT.soc_category", postData.category,
                    "WT.soc_author", postData.author,
                    "WT.soc_login", postData.login,
                    "WT.soc_action", action,
                    "WT.soc_type", actionTarget,
                    "WT.soc_reply_text", replyData.body,
                    "WT.soc_reply_author", replyData.author,
                    "WT.soc_reply_login", replyData.login,
                    "WT.shp_wpv", webpart,
                    "WT.nv", navigation
                ],
                finish: function (dcsObject, multiTrackObject) {
                    sp.finishCleanup(dcsObject, multiTrackObject);
                }
            });
        };

        // Get Blog Post info to use for /EditComments page, which doesn't have access to the post itself
        (function () {
            if (window.location.href.toLowerCase().indexOf(constants.EDIT_COMMENT_PAGE) >= 0) {
                // Extract the post URL from the Source Parameter
                var source = decodeURIComponent(sp.getQueryParamValue("Source", window.location.search));
                if (source) {

                    // Get the post comment item data
                    var getCommentFromREST = function () {
                        // Get the comment id and then the comment data
                        var id = sp.getQueryParamValue("ID", window.location.href);
                        if (id) {
                            request = {
                                url: window.wt_sp_globals.spPageContextInfo.webAbsoluteUrl + "/_api/web/lists/GetByTitle('Comments')/items(" + id + ")/FieldValuesAsText",
                                success: function (data) {
                                    // Get the category and then post data from the comment's PostID property
                                    getCategoryDataFromREST(data.d.PostID);
                                },
                                error: function (err) { }
                            };
                            sp.wtAjax(request, false);
                        }
                    };

                    // Get the post category list data
                    var getCategoryDataFromREST = function (id) {
                        var originalPostId = id;
                        // Get the post categories first, then post data
                        if (originalPostId) {
                            request = {
                                url: window.wt_sp_globals.spPageContextInfo.webAbsoluteUrl + "/_api/web/lists/GetByTitle('Categories')/Items",
                                success: function (data) {
                                    // Convert list of categories into usable objecvt
                                    var categoryList = data.d.results;
                                    var categoryNamesObj = {};
                                    for (var i = 0; i < categoryList.length; i++) {
                                        categoryNamesObj[categoryList[i].ID] = categoryList[i].Title;
                                    }
                                    // Get the post data
                                    getPostDataFromREST(categoryNamesObj, originalPostId);
                                },
                                error: function (err) { }
                            };
                            sp.wtAjax(request, false);
                        }
                    };

                    // Get the post item data
                    var getPostDataFromREST = function (categoryObject, id) {
                        var categoryNames = categoryObject;
                        var originalPostId = id;
                        // Get the post list item data
                        if (originalPostId) {
                            request = {
                                url: window.wt_sp_globals.spPageContextInfo.webAbsoluteUrl + "/_api/web/lists/GetByTitle('Posts')/items(" + originalPostId + ")?$expand=Author&$select=FileDirRef,Title,Body,Author/Title,PostCategoryId",
                                success: function (data) {
                                    // Remove unwanted text from string value
                                    var cleanText = function (text) {
                                        // Remove HTML from body text
                                        text = text.replace(/(<([^>]+)>)/ig, "");
                                        // Convert Ascii to characters
                                        text = sp.convertAsciiToChar(text);
                                        // Trim unwanted whitespace
                                        text = text.replace(/\s+/g, " ").trim();
                                        // Trim to final text length
                                        return trimText(text);
                                    };

                                    // Get list of categories and see if the post is connected to any
                                    var createCategoryList = function (ids) {
                                        var categoryIds = ids;
                                        var categoryList = [];
                                        for (var i = 0; i < categoryIds.length; i++) {
                                            // Cross reference the category name from the category id
                                            if (categoryNames.hasOwnProperty(categoryIds[i])) {
                                                categoryList.push(categoryNames[categoryIds[i]]);
                                            }
                                        }
                                        return categoryList.join(";");
                                    };

                                    // Get main object
                                    var d = data.d;

                                    // Set results for later use
                                    window.wt_sp_globals.blogPostData = {
                                        url: window.location.protocol + "//" + window.location.hostname + d.FileDirRef + constants.POST_PAGE + "?ID=" + originalPostId,
                                        title: cleanText(d.Title),
                                        body: cleanText(d.Body),
                                        author: d.Author.Title,
                                        category: createCategoryList(d.PostCategoryId.results)
                                    };
                                },
                                error: function (err) { }
                            };
                            sp.wtAjax(request, false);
                        }
                    };

                    var request, postId;
                    // Extract the Post ID
                    postId = parseInt(sp.getQueryParamValue("ID", source));
                    if (postId) {
                        // User came from post directly, Post URL and ID is found in Source parameter
                        getCategoryDataFromREST(postId);
                    }
                    else if (source.toLowerCase().indexOf(constants.ALL_COMMENTS_PAGE) >= 0) {
                        // User came from /AllComments.aspx page instead, get comment first, to then get postID
                        getCommentFromREST();
                    }
                }
            }
            else if (window.wt_sp_globals.blogPostData) {
                delete window.wt_sp_globals.blogPostData;
            }
        })();

        // Blog Impressions
        if (impressions) {
            // Initial Post Impressions on Page View - Triggered by Collect event, which should allow this to work on MDS pages as well.
            dcs.addTransform(function (dcsObject, multiTrackObject) {
                try {
                    // Update site type on SPA page loads - can't guarantee which addTransform runs first - so run from all
                    sp.getSocialSiteType();

                    // Check if the page even has a feed
                    if (!sp.isSocialPage()) return;

                    if (!window.wt_sp_globals.modernView) {
                        // Classic View
                        // Check if this is Blog home page
                        if (window.location.href.toLowerCase().indexOf(constants.DEFAULT_PAGE) >= 0) {
                            setTimeout(function () {
                                var posts = document.querySelectorAll("ul.ms-blog-postList > li");
                                for (var i = 0; i < posts.length; i++) {
                                    sendDataBlog(posts[i], null, constants.IMPRESSION, constants.POST, null, null);
                                }
                            }, 250);
                        }
                    }
                }
                catch (ex) { }
            }, 'collect');
        }

        // Like a Post
        dcs.addSelector('.ms-blog-postList a[id^="likesElement-"]', {
            filter: function (dcsObject, options) {
                var e = options.event;
                var el = options.element;
                // Avoid right clicks, disabled links, or double clicks
                if (dcsObject._isRightClick(e) || el.disabled || replyInProgress) return true;
                var text = sp.translateLanguage(sp.getElementText(el));
                var post = getBlogPostContainer(el);
                if (post) {
                    sendDataBlog(post, null, text, constants.POST, sp.getWebPartTitleOnClick(el), (dcsObject ? dcsObject.dcsNavigation(e, dcsObject.navigationtag) : null));
                }
                return true;
            }
        });

        // Email a link for directly click on Blog home page
        dcs.addSelector('.ms-blog-postList a.ms-secondaryCommandLink[id*="-share_"]', {
            filter: function (dcsObject, options) {
                var e = options.event;
                var el = options.element;
                // Avoid right clicks, disabled links, or double clicks
                if (dcsObject._isRightClick(e) || el.disabled || replyInProgress) return true;
                var post = getBlogPostContainer(el);
                if (post) {
                    sendDataBlog(post, null, constants.EMAIL, constants.POST, sp.getWebPartTitleOnClick(el), (dcsObject ? dcsObject.dcsNavigation(e, dcsObject.navigationtag) : null));
                }
                return true;
            }
        });

        // Store reference to the ... menu for ECB "Email a link" clicks - processed with anonymous ExecuteOnClick wrapper below
        dcs.addSelector('.ms-blog-postList a.ms-menu-a.ms-pivotControl-overflowDot', {
            filter: function (dcsObject, options) {
                (setTimeout(function () {
                    window.wt_sp_globals.tempElement = options.element;
                }, 500));               
                return true;
            }
        });

        // Post Comment (Click Post Button)
        dcs.addSelector('button[id*="-CommentSubmit"]', {
            actionElems: { "BUTTON": 1 },
            filter: function (dcsObject, options) {
                var e = options.event;
                var el = options.element;
                // Avoid right clicks, disabled links, or double clicks
                if (dcsObject._isRightClick(e) || el.disabled || replyInProgress) return true;
                var par = sp.getParentByClassNameHas(el, "ms-blog-postList");
                if (par && isValid(par.querySelector("textarea"), constants.REPLY)) {
                    var post = document.querySelector("ul.ms-blog-postList li");
                    var reply = par;
                    sendDataBlog(post, reply, constants.REPLY, constants.POST, sp.getWebPartTitleOnClick(el), (dcsObject ? dcsObject.dcsNavigation(e, dcsObject.navigationtag) : null));
                }
                return true;
            }
        });

        // Lists Edit Posts/Comments Confirmation Button on EditPost.aspx|EditComment.aspx (Click Save/Publish Button when edit)
        dcs.addSelector('input[id*="diidIOSaveItem"],a[id^="Ribbon.PostListForm.Edit.Commit.Publish"],a[id^="Ribbon.ListForm.Edit.Commit.Publish"]', {
            actionElems: { "A": 1, "INPUT": 1 },
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                if (window.location.href.toLowerCase().indexOf(constants.NEW_POST_PAGE) >= 0)
                    return true;

                if (window.location.href.toLowerCase().indexOf(constants.EDIT_POST_PAGE) >= 0) {
                    // Editing a post
                    sendDataBlog(null, null, constants.EDIT, constants.POST, null, null);
                }
                else if (window.location.href.toLowerCase().indexOf(constants.EDIT_COMMENT_PAGE) >= 0) {
                    // Editing a comment
                    sendDataBlog(null, null, constants.EDIT, constants.REPLY, null, null);
                }
                return true;
            }
        });

        // Lists New Posts on NewPost.aspx (Create a Post in Blog tools)
        dcs.addSelector('input[id*="diidIOSaveItem"],a[id="Ribbon.PostListForm.Edit.Commit.Publish-Large"]', {
            actionElems: { "A": 1, "INPUT": 1 },
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                if (window.location.href.toLowerCase().indexOf(constants.EDIT_POST_PAGE) >= 0 ||
                    window.location.href.toLowerCase().indexOf(constants.EDIT_COMMENT_PAGE) >= 0)
                    return true;

                sendDataBlog(null, null, constants.POST, constants.POST, null, null);
                return true;
            }
        });
        
        // Delete Posts on EditPost.aspx
        dcs.addSelector('a[id="Ribbon.PostListForm.Edit.Actions.DeleteItem-Large"],a[id="Ribbon.ListForm.Edit.Actions.DeleteItem-Large"]', {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                if (window.location.href.toLowerCase().indexOf(constants.EDIT_POST_PAGE) >= 0) {
                    sendDataBlog(null, null, constants.DELETE, constants.POST, null, null);
                }
                else if (window.location.href.toLowerCase().indexOf(constants.EDIT_COMMENT_PAGE) >= 0) {
                    sendDataBlog(null, null, constants.DELETE, constants.REPLY, null, null);
                }
                return true;
            }
        });

        /************************************************SHARED*******************************************************/

        // Get the current Feed Name, Feed Type, and Author if avaliable, runs on Page Views
        dcs.addTransform(function (dcsObject, multiTrackObject) {
            try {
                // Update site type on SPA page loads - can't guarantee which addTransform runs first - so run from all
                sp.getSocialSiteType();

                // Check if the page even has a feed
                if (!sp.isSocialPage()) return;

                if (!multiTrackObject.argsa) multiTrackObject.argsa = [];
                feedTitle = (sp.getWebPartTitleOnClick(document.querySelector("#ms-microfeeddiv")) || window.wt_sp_globals.spPageContextInfo.webTitle);
                var url = "";
                var title = "";
                var text = "";
                var category = "";
                var image = false;
                var author = "";
                var login = "";
                var action = "Feed View";
                var type = "";
                var postData;
                if (sp.getQueryParamValue("ThreadID", window.location.search)) {
                    // Newsfeed thread pages
                    postData = getPostData(document.querySelector(".ms-microfeed-thread"));
                    url = window.location.protocol + "//" + postData.dcssip + postData.dcsuri + postData.dcsqry;
                    text = postData.body;
                    image = postData.image;
                    author = postData.author;
                    login = postData.login;
                    action = constants.POST_VIEW;
                    type = constants.POST;
                }
                else if (window.location.href.toLowerCase().indexOf(constants.POST_PAGE) >= 0 && sp.getQueryParamValue("ID", window.location.search)) {
                    // Blog post pages
                    postData = getPostDataBlog(document.querySelector("ul.ms-blog-postList li"));
                    url = window.location.protocol + "//" + postData.dcssip + postData.dcsuri + postData.dcsqry;
                    title = postData.title;
                    text = postData.body;
                    category = postData.category;
                    image = "";
                    author = postData.author;
                    login = postData.login;
                    action = constants.POST_VIEW;
                    type = constants.POST;
                }
                else if (window.location.href.toLowerCase().indexOf(constants.NEW_POST_PAGE) >= 0 ||
                    window.location.href.toLowerCase().indexOf(constants.EDIT_POST_PAGE) >= 0 ||
                    window.location.href.toLowerCase().indexOf(constants.EDIT_COMMENT_PAGE) >= 0) {
                    // Edit/New Post and Comment pages.
                    action = "";
                }
                multiTrackObject.argsa.push(
                    "WT.soc_content", feedTitle,
                    "WT.soc_feed_type", window.wt_sp_globals.socialSiteType,
                    "WT.soc_url", url,
                    "WT.soc_title", title,
                    "WT.soc_text", text,
                    "WT.soc_image", image,
                    "WT.soc_category", category,
                    "WT.soc_author", author,
                    "WT.soc_login", login,
                    "WT.soc_action", action,
                    "WT.soc_type", type
                );
            }
            catch (ex) { }
        }, 'collect');

        // Bind to old style ECB Menu clicks
        (function () {
            if (typeof window.ExecuteOnClick === 'function' && !window.ExecuteOnClick.isSocialWT) {
                // Copy original
                var origExecuteOnClick = window.ExecuteOnClick;
                // Replace original
                window.ExecuteOnClick = function (b, a) {
                    try {
                        var _a = a._arrSelected[0];
                        if (_a) {
                            var text = sp.translateLanguage(sp.getElementText(_a));
                            if (window.wt_sp_globals.tempElement) {
                                if (text === constants.EMAIL) {
                                    // BLOGS
                                    // Send Email a link hit for Blog Posts
                                    var post = getBlogPostContainer(window.wt_sp_globals.tempElement);
                                    if (post) {
                                        sendDataBlog(post, null, constants.EMAIL, constants.POST, sp.getWebPartTitleOnClick(post), null);
                                    }
                                }
                                else if (text.match(new RegExp("(" + constants.FOLLOW + "|" + constants.STOP_FOLLOWING + ") #[0-9A-Z]+", "ig")) && text.match(constants.HASH_FROM_STRING)) {
                                    // NEWSFEEDS
                                    // Send "Follow #hash" ECB Menu link click hits for Newsfeed posts
                                    var action = text.match(/stop/ig) ? constants.UNFOLLOW_HASH_TAG : constants.FOLLOW_HASH_TAG;
                                    var hash = text.match(constants.HASH_FROM_STRING)[0];
                                    // Reply Element
                                    var element = sp.getParentByClassNameHas(window.wt_sp_globals.tempElement, "ms-microfeed-replyBody");
                                    if (!element) {
                                        var par = sp.getParentByClassNameHas(window.wt_sp_globals.tempElement, "ms-microfeed-rootBody");
                                        if (par) {
                                            // Post Element
                                            element = par.querySelector(".ms-microfeed-postBody");
                                        }
                                    }
                                    if (element) {     
                                         // Find matching hash from text box
                                        var anchor;
                                        var links = element.querySelectorAll("a#ms-actorElement.ms-subtleLink");
                                        for (var i = 0; i < links.length; i++) {
                                            if (sp.getElementText(links[i]) === hash) {
                                                anchor = links[i];
                                                var res = sp.getURIArrFromHREF(anchor.href);
                                                Webtrends.multiTrack({
                                                    argsa: [
                                                        "DCS.dcssip", res.dcssip,
                                                        "DCS.dcsuri", res.dcsuri,
                                                        "DCS.dcsqry", res.dcsqry,
                                                        "DCS.dcsref", res.dcsref,
                                                        "WT.ti", text,
                                                        "WT.dl", "SHP",
                                                        "WT.soc_action", action,
                                                        "WT.soc_hash", hash,
                                                        "WT.soc_hash_id", getHashId(anchor)
                                                    ]
                                                });
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    catch (e) { }

                    // Call original function
                    origExecuteOnClick(b, a);
                    return;
                };
                window.ExecuteOnClick.isSocialWT = true;
            }
        })();
    };

    /***************************************************END****************************************************/

    /*************************************************REGISTER**************************************************/

    // TODO: Consider incorporating this entire file into webtrends.sp.js - this section would be unnecessary
    // Register plugin, but wait for dom ready to make sure elements are present, and for sp plugin to be ready
    window.wt_sp_globals.registerSPSocial = function (tag, plugin) {
        // Boolean for dom readiness
        var domReady = false;
        if (document.readyState !== "loading") domReady = true;
        else if (document.addEventListener) document.addEventListener("DOMContentLoaded", function () { domReady = true; } , false);
        else document.attachEvent("DOMContentLoaded", function () { domReady = true; } );

        // Reference helper methods from webtrends.sp.js plugin (sp) for internal use, call worker function
        var waitForSpPlugin = setInterval(function () {
            if (domReady && window.wt_sp_globals.pluginObj) {
                clearInterval(waitForSpPlugin);
                sp = window.wt_sp_globals.pluginObj;
                spsocial(tag, plugin);                
            }
        }, 100);
    };
    window.Webtrends.registerPlugin("spsocial", window.wt_sp_globals.registerSPSocial);
})();
