/*
Copyright 2015-2020 Webtrends
SharePoint User Profile Service Plugin 
Version: 4.15.0.351
*/

window.wt_sp_globals.wtUserProfileService = function (tag, plugin) {

    // Keep track if registerPluginCallback has already been run
    var registered = false;
    var retry = true;

    // the root url of Profile API
    var apiEndPoint = "";

    // Retrieve site URL
    var siteURLCount = 0;

    // Call registerPluginCallback if it is not run
    var wtRegisterPlugin = function () {
        if (registered === false) {
            tag.registerPluginCallback("ups");
            registered = true;
        }
    };

    // Register wt_sp_user namespace in MDS so it isn't lost on page transition
    var wtRegisterNameSpace = function () {
        if (typeof asyncDeltaManager !== "undefined") {
            var waitForType = setInterval(function () {
                if (typeof Type !== "undefined" && typeof Type.registerNamespace === "function" && typeof wt_sp_user !== "undefined") {
                    clearInterval(waitForType);
                    Type.registerNamespace("wt_sp_user");
                }
            }, 500);
        }
    };

    var wtGetProfileInfo = function (tag, plugin) {
        // AJAX function configuration
        wtAjax({
            type: 'GET',
            url: apiEndPoint + "/_api/SP.UserProfiles.PeopleManager/GetMyProperties",
            async: true,
            headers: { Accept: "application/json;odata=verbose" },
            timeout: 15000,
            // On successful request
            success: function (data) {
                if (typeof window.wt_sp_user === 'undefined') window.wt_sp_user = {};

                // Default attributes to collect
                // These are ignored/overwritten by the App
                var wtData = {};

                // Add User Attributes to list from plugin config in webtrends.load.js, removing redundant entries
                if (Webtrends.plugins.ups['userProfile']) {
                    for (var p in Webtrends.plugins.ups['userProfile']) {
                        // If the plugin config list is a User Attribute and not a WT parameter
                        if (p.indexOf("WT.") == -1) {
                            // If the User Attribute is already defined in the default list, remove it
                            for (var q in wtData) {
                                if (wtData[q] == Webtrends.plugins.ups['userProfile'][p]) {
                                    delete wtData[q];
                                }
                            }
                            // Set the User Attribute to the desired WT parameter
                            wtData[p] = Webtrends.plugins.ups['userProfile'][p];
                        }
                    }
                }

                // Store results of AJAX request
                var g = data.d.UserProfileProperties.results;

                // Move the user profile data into a simple array
                // it's easier to deal with this way and saves
                // iterating through it multiple times
                var userProfileDat = [];
                for (var c = 0; c < g.length; c++) {
                    if (g[c].Value && g[c].Value != '') userProfileDat[g[c].Key] = g[c].Value;
                }

                // Collect User Attributes and store in "wt_sp_user"
                for (var k in wtData) {
                    if (typeof userProfileDat[k] != 'undefined') {
                        wt_sp_user[wtData[k]] = userProfileDat[k];
                        // some values are | delimited string values, and we only need the last section
                        if (wt_sp_user[wtData[k]] && wt_sp_user[wtData[k]].split('|').length > 0) {
                            wt_sp_user[wtData[k]] = wt_sp_user[wtData[k]].split("|").pop();
                        }
                    }
                }

                // Replace WT parameter values with alternative User Attributes, or block them from sending
                if (Webtrends.plugins.ups['userProfile']) {
                    for (var p in Webtrends.plugins.ups['userProfile']) {
                        if (p.indexOf("WT.") >= 0) {
                            // if the User Property exists, replace the existing parameter value with its value.
                            if (userProfileDat[Webtrends.plugins.ups['userProfile'][p]] !== undefined)
                                wt_sp_user[p] = userProfileDat[Webtrends.plugins.ups['userProfile'][p]];
                            // if the User Property does not exist, replace the existing parameter value with an empty string to block it from sending
                            else wt_sp_user[p] = "";
                        }
                    }
                }

                // Cache our fetched info to avoid needless AJAX calls
                if (typeof sessionStorage !== 'undefined' && sessionStorage !== null) {
                    sessionStorage.userProfileInfo = JSON.stringify(wt_sp_user);
                }

                // Inform tag of Ready
                wtRegisterNameSpace();
                wtRegisterPlugin();
            },
            error: function (err) {

                // Add error parameter for debugging
                Webtrends.addTransform(function (dcsObject, trackObject) {
                    try {
                        if (!trackObject.argsa) {
                            trackObject.argsa = [];
                        }
                        trackObject.argsa.push(
                            "WT.ups_error", err.toString()
                        );
                    } catch (ex) { }
                }, "collect");

                // Inform tag of Ready
                wtRegisterNameSpace();
                wtRegisterPlugin();
            }
        }, tag, plugin);
    };

    // AJAX request prototype
    var wtAjax = function (func, tag, plugin) {

        var upsReq = new XMLHttpRequest();
        if (typeof func.timeout !== 'undefined') timeoutTime = func.timeout;
        else timeoutTime = 20000;

        ajaxTimeout = setTimeout(function () {
            if (typeof func.error === 'function') {
                func.error('UPS AJAX: Timeout');
            }
        }, timeoutTime);

        upsReq.open(func.type, func.url, true);
        if (func.headers) {
            for (var h in func.headers) {
                upsReq.setRequestHeader(h, func.headers[h]);
            }
        }

        upsReq.onreadystatechange = function () {
            if (upsReq.readyState === 4) {
                // Response Received
                if (upsReq.status === 200) {
                    // Request was successful
                    clearTimeout(ajaxTimeout);
                    func.success(JSON.parse(upsReq.responseText));
                }
                else if (upsReq.status === 401 && retry) {
                    // Retry due to unauthorized access
                    clearTimeout(ajaxTimeout);
                    retry = false;
                    wtGetProfileInfo(tag, plugin);
                }
                else {
                    // Something went wrong
                    clearTimeout(ajaxTimeout);
                    func.error("UPS AJAX: Error " + upsReq.status);
                }
            }
        };
        // Initiate request
        upsReq.send();
    };

    // Poll for element which contains site collection URL
    var siteURL = setInterval(function () {
        if (typeof window.wt_sp_globals.spPageContextInfo !== 'undefined' || siteURLCount >= 40) {
            clearInterval(siteURL);
            if (window.wt_sp_globals.spPageContextInfo.webAbsoluteUrl) {
                apiEndPoint = window.wt_sp_globals.spPageContextInfo.webAbsoluteUrl;
            }
            else {
                // Try to get proper API Endpoint based on URL
                var pathname = "";
                var uri = window.location.pathname.toLowerCase();
                if (uri.length === 0 || (uri.lastIndexOf("/") + 1 !== uri.length && uri.substring(uri.lastIndexOf("/"), uri.length).indexOf(".") === -1)) uri += "/";
                if (uri.indexOf("/_layouts/") >= 0) {
                    pathname = uri.substring(0, uri.indexOf("/_layouts/"));
                }
                else if (uri.indexOf("/sitepages/") >= 0) {
                    pathname = uri.substring(0, uri.indexOf("/sitepages/"));
                }
                else if (uri.indexOf("/pages/") >= 0) {
                    pathname = uri.substring(0, uri.indexOf("/pages/"));
                }
                else if (uri.indexOf("/lists/") >= 0) {
                    pathname = uri.substring(0, uri.indexOf("/lists/"));
                }
                else if (uri.indexOf("/forms/") >= 0) {
                    uri = uri.substring(0, uri.indexOf("/forms/"));
                    pathname = uri.substring(0, uri.lastIndexOf("/"));
                }
                else {
                    pathname = uri.substring(0, uri.lastIndexOf("/"));
                }
                apiEndPoint = window.location.protocol + "//" + window.location.hostname + pathname;
            }
            // If we do not yet have cached copy, fetch user profile info
            if (typeof window.wt_sp_user === 'undefined') {
                if (typeof sessionStorage !== 'undefined' && sessionStorage !== null && sessionStorage.userProfileInfo) {
                    // Assign by extracting from webstorage cache; register plugin immediately
                    window.wt_sp_user = JSON.parse(sessionStorage.userProfileInfo);
                    wtRegisterNameSpace();
                    wtRegisterPlugin();
                }
                else {
                    // Begin AJAX request for User Info; register plugin upon conclusion
                    // If we're in a app only authenticated appweb, we shouldn't do user profile collection
                    if (window.wt_sp_globals.spPageContextInfo.webTemplate == "17" &&
                        tag.config.plugins.sp.skipUPSInApps &&
                        window.location.pathname.toLowerCase().indexOf("webtrendssharepointtaginstaller") === -1 &&
                        window.location.pathname.toLowerCase().indexOf("webtrendsadvancedanalytics") === -1) {
                        wtRegisterPlugin();
                    }
                    else {
                        wtGetProfileInfo(tag, plugin);
                    }
                }
            }
            else {
                // Inform tag of Ready
                wtRegisterNameSpace();
                wtRegisterPlugin();
            }
        }
        else {
            siteURLCount++;
        }
    }, 100);

    // clear userProfileInfo when click "Sign out" in O365 and SP2016
    tag.addSelector('a[id*="_ShellSignout"]', {
        filter: function (dcsObject, options) {
            if (typeof sessionStorage !== 'undefined' && sessionStorage !== null && sessionStorage.userProfileInfo) {
                sessionStorage.removeItem("userProfileInfo");
            }
            return true;
        }
    });

    // clear userProfileInfo when click "Sign out" in SP2013
    tag.addSelector('div#welcomeMenuBox', {
        actionElems: { DIV: 1 },
        filter: function (dcsObject, options) {
            WebTrendsSP_intra.prototype.manualLinkBinder('a#mp1_0_1_Anchor', 'mousedown', function () {
                if (typeof sessionStorage !== 'undefined' && sessionStorage !== null && sessionStorage.userProfileInfo) {
                    sessionStorage.removeItem("userProfileInfo");
                }
            }, 500);
            return true;
        }
    });
};

Webtrends.registerPlugin("ups", window.wt_sp_globals.wtUserProfileService);
